﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Web;
using System.Configuration;
using AFDS.Core;

namespace AFDS.BusinessLogic.Helpers
{
    public static class AFDSHelper
    {
        /// <summary>
        /// Convert a string to Slug format
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public static string ToMakeSlug(string name)
        {
            name = name.Replace(":", string.Empty);
            name = name.Replace("/", string.Empty);
            name = name.Replace("?", string.Empty);
            name = name.Replace("#", string.Empty);
            name = name.Replace("[", string.Empty);
            name = name.Replace("]", string.Empty);
            name = name.Replace("@", string.Empty);
            name = name.Replace(".", string.Empty);
            name = name.Replace(",", string.Empty);
            name = name.Replace("\"", string.Empty);
            name = name.Replace("&", string.Empty);
            name = name.Replace("'", string.Empty);
            name = name.Replace(" ", "-");
            name = name.Replace("--", "-");
            name = name.ToLower();

            return HttpUtility.UrlEncode(name).Replace("%", string.Empty);
        }

        /// <summary>
        /// Returns the virtual path absolute
        /// </summary>
        public static string VirtualPathAbsolute
        {
            get
            {
                return VirtualPathUtility.ToAbsolute(
                        ConfigurationManager.AppSettings["VirtualPath"]);
            }
        }

        /// <summary>
        /// Return the AFDS Directory Name
        /// </summary>
        /// <returns></returns>
        public static string GetDirectoryName()
        {
            string strName = "";
            using (SettingHelper settinghl = new SettingHelper())
            {
                strName = settinghl.GetAll()["DirectoryName"].ToString();
            }
            return strName;
        }


        public static bool SetRootCategory(Category c) {
            using (SettingHelper settinghl = new SettingHelper()) {
                try {
                    Hashtable hs = settinghl.GetAll();
                    hs["Rootcategory"] = c.Id.ToString();
                    settinghl.Save(hs);
                    return true;                
                } catch {
                    return false;
                }
            }
        }

        /// <summary>
        /// Return the Category Root
        /// </summary>
        /// <returns></returns>
        public static Category GetRootCategory()
        {
            Category c = null;
            using (CategoryHelper cHelper = new CategoryHelper())
            {
                try {
                    c = cHelper.GetRootCategory();
                }
                catch (AFDSException ex)
                {
                    if (ex.ErrorType == ErrType.NotFoundRootCategory)
                        c = cHelper.CreateRootCategory();                        
                }
                
            }
            return c;
        }

        /// <summary>
        /// Return the theme name
        /// </summary>
        /// <returns></returns>
        public static string GetThemeName()
        {
            string c = "";
            using (ThemeHelper tHelper = new ThemeHelper())
            {
                c = tHelper.GetCurrentThemeName();
            }
            return c;
        }

        public static Dictionary<Guid, string> GetCbItems(bool onlyAllowSites, bool addRoot) {
                return GetCbItems(onlyAllowSites, addRoot, null);
        }


        /// <summary>
        /// Returns the items for used by combo of categories
        /// </summary>
        /// <param name="onlyAllowSites">Select just categories that allow add sites</param>
        /// <param name="addRoot">Add the root category</param>
        /// <returns></returns>
        public static Dictionary<Guid, string> GetCbItems(bool onlyAllowSites, bool addRoot, string idCurrent)
        {
            Dictionary<Guid, string> items = new Dictionary<Guid, string>();
            Guid RootId = AFDSHelper.GetRootCategory().Id;
            ArrayList iii = new ArrayList();
            CategoryHelper c = new CategoryHelper();
            c.GetCategoriesPathTree(ref iii, "", RootId, onlyAllowSites, addRoot, idCurrent);
            IEnumerator numerator = iii.GetEnumerator();

            while (numerator.MoveNext())
            {
                string[] ss = ((string)numerator.Current).Split(new char[] { '|' });
                items.Add(new Guid(ss[1]), ss[0]);            
            }
            return items;
        }

        public static string StorageLocation()
        {
            if (String.IsNullOrEmpty(System.Web.Configuration.WebConfigurationManager.AppSettings["StorageLocation"]))
                return @"~/app_data/";
            return System.Web.Configuration.WebConfigurationManager.AppSettings["StorageLocation"];

        }

        public static string StorageLocationMapPath()
        {
            return System.Web.HttpContext.Current.Server
               .MapPath(StorageLocation());
        }

        public static int GetCountAllUsers() {
            return System.Web.Security.Membership.GetAllUsers().Count;
        }

        public static string GetAdminstratorRolName() {
            SettingHelper sHelper = new SettingHelper();
            return sHelper.GetAll()["Administratorrole"].ToString();
        }

    }
}
