﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using AFDS.BusinessLogic;
using AFDS.Core.Base;
using AFDS.DataAccess;
using AFDS.DataAccess.Providers;
using AFDS.Core;


namespace AFDS.BusinessLogic.Helpers
{
    public class SiteHelper : HelperBase<SiteHelper, Site>, IDisposable
    {
        
        public override Site Select(Guid id)
        {
            Site s = (Site)ProxyProvider.SelectSite(id);
            //Load Categories in site
            List<Guid> ids_cats = ProxyProvider.GetCategoriesIDBySite(s);
            using(CategoryHelper cHelper = new CategoryHelper()){
                foreach (Guid g in ids_cats) s.Categories.Add(cHelper.Load(g));
            }
            return s;
        }

        public override void Insert(Site item)
        {
            if (ValidateSite(item)) {
                AFDS.Core.Site.OnSaving(item, new SiteSavingEventArgs(item));
                ProxyProvider.InsertSite(item as ISite);
            } else {
                throw new
                    AFDSException(AFDS.Core.SeverityLevel.Information,
                    LogLevel.Debug,
                    "This site is already on the category",
                    ErrType.InsertASite);  
            }
        }

        public override void Update(Site item)
        {
            if (Exist(item))
            {
                AFDS.Core.Site.OnSaving(item, new SiteSavingEventArgs(item));
                ProxyProvider.UpdateSite(item as ISite);
            }
            else
            {
                Insert(item);
            } 
        }

        public override void Delete(Site item)
        {
            ProxyProvider.DeleteSite(item as ISite);
        }

        public override List<Site> GetAll()
        {
            //pasar esto a capa de datos
            string folder = AFDS.DataAccess.Utils.StorageLocationMapPath() + "Sites" + "/";
            List<Site> sites = new List<Site>();

            foreach (string file in Directory.GetFiles(folder, "*.xml", SearchOption.TopDirectoryOnly))
            {
                FileInfo info = new FileInfo(file);
                string id = info.Name.Replace(".xml", string.Empty);

                Site site = base.Load(new Guid(id));

                sites.Add(site);
            }            
            return sites;
        }

        public List<Site> GetTopAndOrderByDate(int iTop) {
            List<Site> _items = GetAll();

            return (from t in _items
                    orderby t.DateCreated descending
                    select t).Take(iTop).ToList<Site>();
        }

        public List<Site> GetAllOrderByDate()
        {
            List<Site> _items = GetAll();

            return (from t in _items
                    orderby t.DateCreated descending
                    select t).ToList<Site>();
        }

        public List<Site> GetSitesByUser(string username, string field, SortFactor sortFactor, bool approved) {
            return GetSitesByUser(username, field, sortFactor)
                .FindAll(a => a.Approved == approved);
        }

        public List<Site> GetSitesByUser(string username, string field, SortFactor sortFactor) { 
            System.Web.Security.MembershipUser 
                user = System.Web.Security.Membership.GetUser(username);

            return GetAllOrderBy(field, sortFactor)
                .FindAll(a => a.EMail == user.Email);                        
        }

        public List<Site> GetAllOrderBy(string field, SortFactor sortFactor, bool approved) {
            return GetAllOrderBy(field, sortFactor)
                .FindAll(a => a.Approved == approved);
        }

        public List<Site> GetAllOrderBy(string field, SortFactor sortFactor) {
            List<Site> _Allitems = GetAll();
            List<Site> _items = new List<Site>();

            switch (field){
                case "date":
                    if (sortFactor == SortFactor.DESC)
                        _items = (from t in GetAll()
                                      orderby t.DateCreated descending
                                      select t).ToList<Site>();
                    else
                        _items = (from t in GetAll()
                                  orderby t.DateCreated ascending
                                  select t).ToList<Site>();
                    break;
                case "name":
                    if (sortFactor == SortFactor.DESC)
                        _items = (from t in GetAll()
                                  orderby t.Name descending
                                  select t).ToList<Site>();
                    else
                        _items = (from t in GetAll()
                                  orderby t.Name ascending
                                  select t).ToList<Site>();
                    break;
                case "url":
                    if (sortFactor == SortFactor.DESC)
                        _items = (from t in GetAll()
                                  orderby t.URL descending
                                  select t).ToList<Site>();
                    else
                        _items = (from t in GetAll()
                                  orderby t.URL ascending
                                  select t).ToList<Site>();
                    break;

                case "category":
                    if (sortFactor == SortFactor.DESC)
                        _items = (List<Site>)GetAll().OrderByDescending(a => a.Categories[0].Name).ToList<Site>();
                    else

                        _items = GetAll().OrderBy(a => a.Categories[0].Name).ToList<Site>();
                    break;

            }


            return _items;
        }

        public List<Site> AprovedSites {
            get
            {
                return GetAll().FindAll(s => s.Approved == true);
            }
        }

        public List<Site> NotAprovedSites
        {
            get
            {
                return GetAll().FindAll(s => s.Approved == false);
            }
        }

        public List<Site> GetAllSitesByCategory(ICategory cat) {
            return GetAll().FindAll(
                      s =>
                        s.Categories.Exists(delegate(ICategory c) {
                          return c.Id == cat.Id;
                      })
                );
        }


        public List<Site> GetSitesByCategory(ICategory cat) {                                                                   
            return AprovedSites.FindAll(
                      s =>
                        s.Categories.Exists(delegate (ICategory c){
                            return c.Id == cat.Id;
                        })                        
                );
        }

        public List<Site> GetSitesByCategorySorted(ICategory cat)
        {
            var SortedSites = from s in GetSitesByCategory(cat)
                              orderby s.Score descending
                              select s;

            return SortedSites.ToList<Site>();
        }


        public List<Site> GetSitesByURL(string url)
        {
            return GetAll().FindAll(s => s.URL.ToUpper() == url.ToUpper());
        }


        public Site GetSiteByID(Guid id)
        {
            return GetAll().Find(s => s.Id == id);

        }

        public bool Exist(ISite s)
        {
            ISite st = GetSiteByID(s.Id);
            if (st == null) return false;

            return true;
        }

        public bool ValidateSite(Site item) { 
            foreach(Category i in item.Categories){
                List<Site> sites = this.GetAllSitesByCategory(i as ICategory);
                foreach(Site s in sites){
                    if (s.URL.ToUpper() == item.URL.ToUpper()) return false;                    
                }
            }
            return true;
        }

        /// <summary>
        /// Adds a rating to the Site.
        /// </summary>
        public void Rate(Site site, int rating) {
            if (site.Raters > 0) {
                float total = site.Raters * site.Score;
                total += rating;
                site.Score = (float)(total / site.Raters);
                site.Raters++;
            } else {
                site.Raters = 1;
                site.Score = rating;
            }
            Update(site);
        }


        #region IDisposable Members

        public void Dispose()
        {
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}

