﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

//Based on: http://forums.asp.net/t/1140902.aspx

using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Configuration.Provider;
using System.Collections.Specialized;
using AFDS.BusinessLogic.Helpers;
using AFDS.Core;


namespace AFDS.Controls
{
    public class AFDSSiteMapPath : StaticSiteMapProvider
    {

        private readonly object siteMapLock = new object();
        private SiteMapNode root = null;
        private List<Category> categories;

        public override SiteMapNode BuildSiteMap()
        {
            lock (siteMapLock)
            {
                if (root != null)
                    return root;

                // Clear out the current site map structure
                base.Clear();

                // Get the categories
                using (CategoryHelper cHelper = new CategoryHelper()){
                    categories = cHelper.GetAll();                
                 
                    // Create the root SiteMapNode

                    Category RootCategory = cHelper.GetRootCategory();

                    root = new SiteMapNode(this, RootCategory.Id.ToString(), "~/", RootCategory.Name);
                    AddNode(root);

                    // Create SiteMapNodes for the categories 
                    foreach (Category cat in categories)
                    {

                        SiteMapNode categoryNode = FindSiteMapNodeFromKey(cat.Id.ToString());

                        // Add the category SiteMapNode if it does not exist
                        if (categoryNode == null)
                        {
                            categoryNode = new SiteMapNode(this, 
                                                        cat.Id.ToString(),
                                                        "~/category/" + cat.Slug, 
                                                        cat.Name);

                            SiteMapNode categoryParentNode;
                            

                            //ver esto
                            if (cat.ParentId.ToString() == "00000000-0000-0000-0000-000000000000")
                            {
                                categoryParentNode = root;
                            }else{
                                categoryParentNode = FindSiteMapNodeFromKey(cat.ParentId.ToString());
                                if (categoryParentNode == null)
                                {
                                    Category parentCat = cHelper.Load(cat.ParentId);
                                    categoryParentNode = new SiteMapNode(this,
                                                                parentCat.Id.ToString(),
                                                                "~/category/" + parentCat.Slug,
                                                                parentCat.Name);
                                }


                            }                            
                            AddNode(categoryNode, categoryParentNode);
                        }
                    }
                }

                // Finally, return the root node
                return root;
            }
        }

        protected override SiteMapNode GetRootNodeCore()
        {
            return BuildSiteMap();
        }

    }
}
