﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

//Based from: http://www.martinwilley.com/net/code/imgthumbhelper.html

using System;
using System.IO;
using System.Web;
using System.Drawing;
using System.Drawing.Imaging;

namespace AFDS.Controls.Handlers
{
    public class ImageHandler : IHttpHandler
    {
        #region IHttpHandler Members

        /// <summary>
        /// Gets a value indicating whether another request can use the <see cref="T:System.Web.IHttpHandler"></see> instance.
        /// </summary>
        /// <value></value>
        /// <returns>true if the <see cref="T:System.Web.IHttpHandler"></see> instance is reusable; otherwise, false.</returns>
        public bool IsReusable
        {
            get { return false; }
        }

        /// <summary>
        /// Enables processing of HTTP Web requests by a custom HttpHandler that 
        /// implements the <see cref="T:System.Web.IHttpHandler"></see> interface.
        /// </summary>
        /// <param name="context">An <see cref="T:System.Web.HttpContext"></see> object 
        /// that provides references to the intrinsic server objects 
        /// (for example, Request, Response, Session, and Server) used to service HTTP requests.
        /// </param>
        public void ProcessRequest(HttpContext context)
        {

            if (!string.IsNullOrEmpty(context.Request.QueryString["picture"]))
            {
                string fileName = context.Request.QueryString["picture"];
                int iwidth = int.Parse(context.Request.QueryString["width"]);

                try
                {
                    string folder = "~/App_Data/thumbnail/";
                    FileInfo fi = new FileInfo(context.Server.MapPath(folder) + fileName);

                    if (fi.Exists)
                    {

                        WriteThumbnailSimple(context, fi.FullName.ToString(), iwidth);
                    }
                    else
                    {

                        context.Response.Status = "404 Bad Request";
                    }
                }
                catch (Exception ex)
                {
                    context.Response.Status = "404 Bad Request";
                }
            }
        }

        #endregion


        public static void WriteThumbnailSimple(HttpContext context, string filename, int width)
        {
            using (Image imgThumb = CreateThumbnailSimple(filename, width))
            {
                HttpContext.Current.Response.ContentType = "images/jpeg";
                imgThumb.Save(context.Response.OutputStream, ImageFormat.Jpeg);
            }
        }


        public static Image CreateThumbnailSimple(string filename, int width)
        {
            Image imgThumb;
            //get bitmap from file (see also GetBitmapFromEmbeddedResource)
            using (Bitmap bitmap = GetBitmapFromFile(filename))
            {
                //create a callback delegate
                Image.GetThumbnailImageAbort myCallback = delegate { return false; };

                //preserve aspect ratio
                decimal aspectRatio = (decimal)width / bitmap.Width;
                if (aspectRatio < 0) aspectRatio = 1;
                int thumbnailHeight = Convert.ToInt32(bitmap.Height * aspectRatio);
                if (thumbnailHeight > (width * 2))
                {
                    thumbnailHeight = width * 2;
                    aspectRatio = (decimal)thumbnailHeight / bitmap.Height;
                    width = Convert.ToInt32(bitmap.Width * aspectRatio);
                }

                //images with thumbnails (from most digital cameras) have embedded thumbnails so scale badly
                //rotating 360 apparently kills the embedded thumbnail
                //source: http://aspnet.4guysfromrolla.com/articles/012203-1.2.aspx
                bitmap.RotateFlip(RotateFlipType.Rotate180FlipNone);
                bitmap.RotateFlip(RotateFlipType.Rotate180FlipNone);
                //grab the thumbail from the image
                imgThumb = bitmap.GetThumbnailImage(width, thumbnailHeight, myCallback, IntPtr.Zero);
            }
            return imgThumb;
        }

        public static Bitmap GetBitmapFromFile(string filename)
        {
            return new Bitmap(filename);
        }


    }
}
