﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.Collections.Generic;
using AFDS.BusinessLogic;
using AFDS.BusinessLogic.Helpers;
using System.Drawing;
using System.Drawing.Imaging;
using AFDS.Controls.SiteThumbnail;
using AFDS.Core;


namespace AFDS.Controls.Web
{
    public class AddSiteControl : UserControl
    {
        private Category selCategory;
        private Button btn;
        private Label lbl_title;
        private Label lbl_url;
        private Label lbl_desc;
        private Label lbl_name;
        private Label lbl_lastname;
        private Label lbl_email;
        private Label lbl_cat;
        private Label lbl_headtitle;
        private Label lbl_message;
        private TextBox txt_title;
        private TextBox txt_url;
        private TextBox txt_desc;
        private TextBox txt_name;
        private TextBox txt_lastname;
        private TextBox txt_email;
        private TextBox txt_captcha;
        private Panel addpanel;

        private DropDownList DropDownCategories;

        protected override void OnInit(EventArgs e)
        {            
            base.OnInit(e);
            setObjs();
            btn.Click += new EventHandler(btn_Click);            
        }

        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                ProcessQuerystrings();
                DropDownLoadValues();
            }
        }

        private void ProcessQuerystrings()
        {
            if (!String.IsNullOrEmpty(Request.QueryString["cattoadd"]))
            {
                Guid gCatId = new Guid(Request.QueryString["cattoadd"].ToString());
                using (CategoryHelper hCat = new CategoryHelper())
                {
                    selCategory = hCat.Load(gCatId);
                    if (!selCategory.AllowSites)
                        Response.Redirect("~/default.aspx");
                }
            }
        }

        private void DropDownLoadValues() { 
            CategoryHelper cHelper = new CategoryHelper();
            Category root = cHelper.GetRootCategory();
            ArrayList iii = new ArrayList();

            iii.Add(root.Name + " >|" + root.Id.ToString());
            cHelper.GetCategoriesPathTree(ref iii, "", root.Id, false, true, null);
            iii.Sort();

            IEnumerator numerator = iii.GetEnumerator();
            while (numerator.MoveNext()) {
                string[] ss = ((string)numerator.Current).Split(new char[] {'|'});;
                ListItem item = new ListItem();
                item.Text = ss[0];
                item.Value = ss[1];
                if (ss[1] == selCategory.Id.ToString())
                    item.Selected = true;
                DropDownCategories.Items.Add(item);                
            }          
        }

        private void setObjs()
        {
            btn = (Button)this.FindControl("btn_send");
            lbl_title = (Label)this.FindControl("lbl_title");
            lbl_url = (Label)this.FindControl("lbl_url");
            lbl_desc = (Label)this.FindControl("lbl_desc");
            lbl_name = (Label)this.FindControl("lbl_name");
            lbl_lastname = (Label)this.FindControl("lbl_lastname");
            lbl_email = (Label)this.FindControl("lbl_email");
            lbl_cat = (Label)this.FindControl("lbl_cat");
            txt_title = (TextBox)this.FindControl("txt_title");
            txt_url = (TextBox)this.FindControl("txt_url");
            txt_desc = (TextBox)this.FindControl("txt_desc");
            txt_name = (TextBox)this.FindControl("txt_name");
            txt_lastname = (TextBox)this.FindControl("txt_lastname");
            txt_email = (TextBox)this.FindControl("txt_email");
            txt_captcha = (TextBox)this.FindControl("txt_captcha");
            

            lbl_headtitle = (Label)this.FindControl("lbl_headtitle");
            lbl_message = (Label)this.FindControl("lbl_message");

            addpanel = (Panel)this.FindControl("addpanel");

            DropDownCategories = (DropDownList)this.FindControl("DropDownCategories");

            //Ver multiidioma
            btn.Text = "Send";
            lbl_title.Text = "Title";
            lbl_url.Text = "URL";
            lbl_cat.Text = "Category";
            lbl_desc.Text = "Description";
            lbl_name.Text = "First Name";
            lbl_lastname.Text = "Last Name";
            lbl_email.Text = "Your Email";

            lbl_headtitle.Text = "Add your site";
            
        }

        void btn_Click(object sender, EventArgs e)
        {
            try
            {
                if (txt_captcha.Text.ToString() != Session["RandomNumber"].ToString())
                {
                    lbl_message.Text = "The letters you typed don't match the letters that were shown in the picture.";
                    return;
                }

                using (SiteHelper sHelper = new SiteHelper())
                {
                    if (sHelper.GetSitesByURL(txt_url.Text).Count > 0)
                    {
                        lbl_message.Text = txt_url.Text + " exist on db";
                        addpanel.Visible = false;
                        return;
                    }
                } 

                Guid idSelected = new Guid(DropDownCategories.SelectedValue);
                //Response.Write(idSelected.ToString());

                Site site = new Site();
                site.Id = Guid.NewGuid();
                site.Name = txt_title.Text;
                site.Description = txt_desc.Text;
                site.URL = txt_url.Text;
                site.DateCreated = DateTime.Now;
                site.FirstName = txt_name.Text;
                site.LastName = txt_lastname.Text;
                site.EMail = txt_email.Text;
                site.Approved = false;                

                ImageHelper hImage = new ImageHelper();
                site.Image = hImage.GenerateImage(Server.MapPath("~") + "\\App_Data\\thumbnail\\", site);

                using (CategoryHelper chelper = new CategoryHelper())
                {
                    site.Categories.Add(chelper.Load(idSelected));
                }
                using (SiteHelper sHelper = new SiteHelper())
                {
                    sHelper.Insert(site);
                }
                lbl_message.Text = "Thanks!. Your site was added!";

                addpanel.Visible = false;                 

            }
            catch {
                lbl_message.Text = "Error!";
            }                        
        }
    }
}
