﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;
using System.Web.Configuration;
using System.Configuration.Provider;
using System.Configuration;
using AFDS.DataAccess.Providers.Sections;
using AFDS.Core;
using AFDS.Core.Base;

namespace AFDS.DataAccess.Providers
{
    public static class ProxyProvider
    {
        #region Vars
        //CategoryProviderBase vars
        private static CategoryProviderBase _CategoryProviderBase;
        private static CategoryProviderCollection _category_providers;

        //SettingProviderBase vars
        private static SettingProviderBase _SettingProviderBase;
        private static SettingProviderCollection _setting_providers;

        //SiteProviderBase vars
        private static SiteProviderBase _SiteProviderBase;
        private static SiteProviderCollection _site_providers;

        private static object _lock = new object(); 
        #endregion

        #region Site Provider Loader
        private static void LoadSiteProviders()
        {
            if (_site_providers == null)
            {
                lock (_lock)
                {
                    if (_site_providers == null)
                    {
                        SiteProviderSection section = (SiteProviderSection)WebConfigurationManager.GetSection("AFDS/SiteProviders");

                        // Load registered providers and point _provider
                        // to the default provider
                        _site_providers = new SiteProviderCollection();
                        ProvidersHelper.InstantiateProviders(section.Providers, _site_providers, typeof(SiteProviderBase));
                        _SiteProviderBase = _site_providers[section.DefaultProvider];

                        if (_SiteProviderBase == null)
                            throw new ProviderException("Unable to load default SiteProvider");
                    }
                }
            }
        }

        #endregion


        #region Category Provider Loader
        private static void LoadCategoryProviders()
        {
            if (_category_providers == null)
            {
                lock (_lock)
                {
                    if (_category_providers == null)
                    {
                        // Get a reference to the <blogProvider> section
                        CategoryProviderSection section = (CategoryProviderSection)WebConfigurationManager.GetSection("AFDS/CategoryProviders");

                        // Load registered providers and point _provider
                        // to the default provider
                        _category_providers = new CategoryProviderCollection();
                        ProvidersHelper.InstantiateProviders(section.Providers, _category_providers, typeof(CategoryProviderBase));
                        _CategoryProviderBase = _category_providers[section.DefaultProvider];

                        if (_CategoryProviderBase == null)
                            throw new ProviderException("Unable to load default CategoryProvider");
                    }
                }
            }
        }

        #endregion

        #region Setting Provider Loader
        private static void LoadSettingProviders()
        {
            if (_setting_providers == null)
            {
                lock (_lock)
                {
                    if (_setting_providers == null)
                    {
                        // Get a reference to the <blogProvider> section
                        SettingProviderSection section = (SettingProviderSection)WebConfigurationManager.GetSection("AFDS/SettingProviders");

                        // Load registered providers and point _provider
                        // to the default provider
                        _setting_providers = new SettingProviderCollection();
                        ProvidersHelper.InstantiateProviders(section.Providers, _setting_providers, typeof(SettingProviderBase));
                        _SettingProviderBase = _setting_providers[section.DefaultProvider];

                        if (_SettingProviderBase == null)
                            throw new ProviderException("Unable to load default SettingProvider");
                    }
                }
            }
        }

        #endregion

        #region Site Provider Methods
        public static ISite SelectSite(Guid id)
        {
            LoadSiteProviders();
            return _SiteProviderBase.Select(id);
        }

        public static void UpdateSite(ISite site)
        {
            LoadSiteProviders();
            _SiteProviderBase.Update(site);
        }


        public static void InsertSite(ISite site)
        {
            LoadSiteProviders();
            _SiteProviderBase.Insert(site);
        }

        public static void DeleteSite(ISite site)
        {
            LoadSiteProviders();
            _SiteProviderBase.Delete(site);
        }

        public static List<Guid> GetCategoriesIDBySite(ISite site) {
            LoadSiteProviders();
            return _SiteProviderBase.GetCategoriesIDBySite(site);
        }

        #endregion

        #region Category Provider Methods
        public static ICategory SelectCategory(Guid id)
        {
            LoadCategoryProviders();
            return _CategoryProviderBase.Select(id);
        }

        public static void UpdateCategory(ICategory category)
        {
            LoadCategoryProviders();
            _CategoryProviderBase.Update(category);
        }


        public static void InsertCategory(ICategory category)
        {
            LoadCategoryProviders();
            _CategoryProviderBase.Insert(category);
        }


        public static void DeleteCategory(ICategory category)
        {
            LoadCategoryProviders();
            _CategoryProviderBase.Delete(category);
        }

        #endregion

        #region Setting Provider Methods
        public static Hashtable GetSettings()
        {
            LoadSettingProviders();
            return _SettingProviderBase.GetSettings();
        }

        public static void SaveSettings(Hashtable settings)
        {
            LoadSettingProviders();
            _SettingProviderBase.SaveSettings(settings);
        }
        #endregion

    }
}
