﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Collections.Generic;
using System.Linq;
using System.Xml.Linq;
using System.Text;
using AFDS.Core;
using AFDS.Core.Base;
using System.Collections;
using System.IO;

namespace AFDS.DataAccess.Providers.xml
{
    public partial class XmlCategoryProvider : CategoryProviderBase
    {

        public override ICategory Select(Guid id)
        {
            string folder = Utils.StorageLocationMapPath() + "Categories" + "/";
            string filename =
                folder + id.ToString() + ".xml";
            XElement loaded = XElement.Load(filename);

            Category cat = new Category();
            cat.Id = id;
            cat.Name = loaded.Element("Name").Value;
            cat.Description = loaded.Element("Description").Value;
            cat.AllowSites = bool.Parse(loaded.Element("AllowSites").Value);
            cat.Enable = bool.Parse(loaded.Element("Enable").Value);
            try { cat.ParentId = new Guid(loaded.Element("ParentId").Value); } catch { /*throw new Exception("Error in Select Function");*/ }
            cat.Slug = loaded.Element("Slug").Value;

            return cat as ICategory;
        }

        public override List<ICategory> GetAll()
        {
            /*string folder = Utils.StorageLocationMapPath() + "Categories" + "/";
            List<Category> categories = new List<Category>();            

            foreach (string file in Directory.GetFiles(folder, "*.xml", SearchOption.TopDirectoryOnly))
            {
                FileInfo info = new FileInfo(file);
                string id = info.Name.Replace(".xml", string.Empty);
                Category cat =
                categories.Add(post);
            }

            categories.Sort();
            return categories;*/
            return null;
        }

        public override void Insert(ICategory category)
        {
            string folder = Utils.StorageLocationMapPath() + "Categories" + "/";
            if (!Directory.Exists(folder))
                Directory.CreateDirectory(folder);

            string fileName = folder + category.Id.ToString() + ".xml";

            XDocument xmldocument = new XDocument();
            xmldocument.Declaration = new XDeclaration("1", "utf-8", "yes");

            XElement[] xobjs = new XElement[6];
            xobjs[0] = new XElement("Name", category.Name);
            xobjs[1] = new XElement("Description", category.Description);
            xobjs[2] = new XElement("Enable", category.Enable.ToString());
            xobjs[3] = new XElement("AllowSites", category.AllowSites);
            xobjs[4] = new XElement("ParentId", category.ParentId.ToString());
            xobjs[5] = new XElement("Slug", category.Slug.ToString());

            xmldocument.Add(new XElement("Category", xobjs));
            xmldocument.Save(fileName);
        }

        public override void Update(ICategory category)
        {
            Delete(category);
            Insert(category);
        }

        public override void Delete(ICategory category)
        {
            string folder = Utils.StorageLocationMapPath() + "Categories" + "/";
            string filename =
                folder + category.Id.ToString() + ".xml";
            if (File.Exists(filename))
                File.Delete(filename);
        }

        public override void Dispose()
        {
            GC.SuppressFinalize(this);
        }
    }
}
