﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using System.Collections;
using AFDS.BusinessLogic;
using AFDS.BusinessLogic.Helpers;
using AFDS.Core;
using AFDS.Controls;
using System.Web.Security;

namespace AFDS_WEB.Controllers
{
    [Authorize(Roles = "Administrators, Users")]
    [HandleError]
    public class AFDSAdminController : Controller
    {

        public ActionResult Index()
        {
            if (Roles.IsUserInRole(AFDSHelper.GetAdminstratorRolName())) {
                using (SiteHelper sHelper = new SiteHelper()) {
                    ViewData["Sites"] = sHelper.GetTopAndOrderByDate(10);
                }
                using (CategoryHelper cHelper = new CategoryHelper()) {
                    ViewData["Categories"] = cHelper.GetTop(10);
                }

                ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
                return View();
            } else {
                return View("indexx-foruser");            
            }
        }



        /*Category*/
        [Authorize(Roles = "Administrators")]
        public ActionResult CategoriesList()
        {
            using (CategoryHelper cHelper = new CategoryHelper())
            {
                ViewData["Categories"] = cHelper.GetAll();
            }

            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
            ViewData["Title"] = "Categories";
            return View();
        }


        private IDictionary<string, string> GetTreeCategories()
        {
            Dictionary<string, string> cats = new Dictionary<string, string>();
            List<Category> cs;
            using (CategoryHelper cHelper = new CategoryHelper())
            {
                cs = cHelper.GetAll();
            }

            int i = 0;
            foreach (Category c in cs)
            {
                cats.Add(i.ToString(), c.Name);
                i++;
            }
            return cats;
        }

        [Authorize(Roles = "Administrators")]
        public ActionResult EditCategoryForm(string type, string id)
        {
            Guid idParent = Guid.Empty;
            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();

            if (type.ToUpper() == "ADD")
            {
                ViewData["Category"] = new Category();
                ViewData["Title"] = "Add new Category";
                ViewData["actionType"] = "Add";
            }
            else
            {
                Guid Id = new Guid(id);
                using (CategoryHelper cHelper = new CategoryHelper())
                {
                    ViewData["Category"] = cHelper.GetCategoryByID(Id);

                    idParent = ((Category)ViewData["Category"]).ParentId;
                    if (type.ToUpper() == "DELETE")
                    {
                        ViewData["Title"] = "Delete Category";
                        ViewData["actionType"] = "Delete";
                    }
                    else
                    {
                        ViewData["Title"] = "Edit Category";
                        ViewData["actionType"] = "Update";
                    }
                }
            }

            var selectList = new SelectList(AFDSHelper.GetCbItems(false, true, id), "Key", "Value", idParent);
            ViewData["cbCategories"] = selectList;
            return View();
        }


        [Authorize(Roles = "Administrators")]
        public ActionResult EditCategory()
        {
            Guid id;
            if (string.IsNullOrEmpty(Request.Form["id"]) || Request.Form["id"].ToString() == "0")
                id = Guid.NewGuid();
            else
                id = new Guid(Request.Form["id"]);

            var name = Request.Form["name"];
            var description = Request.Form["description"];
            var slug = Request.Form["slug"];
            var actionType = Request.Form["actionType"];
            var isEnabled = ((Request.Form["IsEnabled"] ?? "").Trim().Equals("on", StringComparison.CurrentCultureIgnoreCase));
            var AllowSites = ((Request.Form["AllowSites"] ?? "").Trim().Equals("on", StringComparison.CurrentCultureIgnoreCase));

            Guid idCatSelected = string.IsNullOrEmpty(Request.Form["Key"]) ? Guid.Empty : new Guid(Request.Form["Key"]);

            using (CategoryHelper cHelper = new CategoryHelper())
            {
                Category category = cHelper.GetCategoryByID(id);
                if (actionType.ToUpper() != "DELETE")
                {
                    if (category == null) { category = new Category(); }
                    category.Id = id;
                    category.Name = name;
                    category.Description = description;
                    category.ParentId = idCatSelected;
                    category.Slug = slug;
                    category.Enable = isEnabled;
                    category.AllowSites = AllowSites;
                    cHelper.Update(category);
                }
                else
                {
                    cHelper.Delete(category);
                }
            }
            return RedirectToAction("CategoriesList");
        }


        /*Site*/

        public ActionResult SitesList()
        {
            //Security
            bool isAdministrator = Roles.IsUserInRole(AFDSHelper.GetAdminstratorRolName());
            string userName = HttpContext.User.Identity.Name;

            //Load collection for Combo OrderBy
            Dictionary<string, string> OrderByItems = new Dictionary<string, string>();
            OrderByItems.Add("date", "Date Created");
            OrderByItems.Add("name", "Site Name");
            OrderByItems.Add("url", "URL");
            OrderByItems.Add("category", "Category");

            //Load collection for Combo FilterItems
            Dictionary<string, string> FilterItems = new Dictionary<string, string>();
            FilterItems.Add("all", "Show All Data");
            FilterItems.Add("approved", "Show Approved Sites");
            FilterItems.Add("notapproved", "Show Not Approved Sites");                      

            var OrderBySelect = Request.Form["OrderBySelect"];
            var FilterSelect = Request.Form["FilterSelect"];

            ViewData["SortFactor"] = Request.Form["SortFactor"] == "ascending" ? SortFactor.ASC : SortFactor.DESC;

            using(SiteHelper sHelper = new SiteHelper()){
                if (!string.IsNullOrEmpty(OrderBySelect) && !string.IsNullOrEmpty(FilterSelect)){

                    //Apply filters
                    switch (FilterSelect) { 
                        case "all" :
                            ViewData["Sites"] = isAdministrator ?
                                sHelper.GetAllOrderBy(OrderBySelect, (SortFactor)ViewData["SortFactor"]) :
                                sHelper.GetSitesByUser(userName, OrderBySelect, (SortFactor)ViewData["SortFactor"]);
                            break;
                        case "approved":
                            ViewData["Sites"] = isAdministrator ?
                                sHelper.GetAllOrderBy(OrderBySelect, (SortFactor)ViewData["SortFactor"], true) :
                                sHelper.GetSitesByUser(userName, OrderBySelect, (SortFactor)ViewData["SortFactor"], true);
                            break;
                        case "notapproved":
                            ViewData["Sites"] = isAdministrator ?
                                sHelper.GetAllOrderBy(OrderBySelect, (SortFactor)ViewData["SortFactor"], false) :
                                sHelper.GetSitesByUser(userName, OrderBySelect, (SortFactor)ViewData["SortFactor"], false);
                            break;               
                    }

                    //State of controls
                    ViewData["OrderByItems"] =
                        new SelectList(OrderByItems, "Key", "Value", OrderBySelect);
                    ViewData["FilterItems"] =
                        new SelectList(FilterItems, "Key", "Value", FilterSelect);
                }
                else{
                    //Default: Order By Date (All data)
                    ViewData["Sites"] = isAdministrator ?
                        sHelper.GetAllOrderByDate() :
                        sHelper.GetSitesByUser(userName, "date", SortFactor.DESC);                    

                    //State of controls
                    ViewData["OrderByItems"] =
                        new SelectList(OrderByItems, "Key", "Value");
                    ViewData["FilterItems"] =
                        new SelectList(FilterItems, "Key", "Value");
                }
            }
            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
            ViewData["Title"] = "Sites";
            return View();
        }



        public ActionResult EditSiteForm(string type, string id)
        {
            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
            bool isAdminiatrator = Roles.IsUserInRole(AFDSHelper.GetAdminstratorRolName());
            ViewData["cbCategories"] = new SelectList(AFDSHelper.GetCbItems(false, true), "Key", "Value", Guid.Empty);
            ViewResult v = View();

            if (type.ToUpper() == "ADD")
            {
                ViewData["Site"] = new Site();
                ViewData["Title"] = "Add new Site";
                ViewData["actionType"] = "Add";
            }
            else
            {
                Guid Id = new Guid(id);
                using (SiteHelper sHelper = new SiteHelper())
                {
                    Site s = sHelper.GetSiteByID(Id);
                    ViewData["Site"] = s;

                    if (type.ToUpper() == "DELETE")
                    {
                        ViewData["Title"] = "Delete Site";
                        ViewData["actionType"] = "Delete";
                    }
                    else
                    {
                        ViewData["Title"] = "Edit Site";
                        ViewData["actionType"] = "Update";
                    }
                    if (s.Categories.Count > 0)
                    {
                        ViewData["cbCategories"] = new SelectList(AFDSHelper.GetCbItems(false, true), "Key", "Value", ((Category)s.Categories[0]).Id);
                        v = isAdminiatrator ? View() : View("EditSiteForm-foruser");
                    }
                }
            }            
            return v;
        }



        public ActionResult CreateSite()
        {
            return RedirectToAction("EditSiteForm");
        }

        public ActionResult EditSiteByUser() {
            Guid id = new Guid(Request.Form["id"]);

            var name = Request.Form["name"];
            var description = Request.Form["description"];
            var url = Request.Form["url"];

            using (SiteHelper sHelper = new SiteHelper()) {
                Site site = sHelper.GetSiteByID(id);
                ImageHelper hImage = new ImageHelper();
                string thumbDirPath = Server.MapPath("~") + "\\App_Data\\thumbnail\\";

                site.Id = id;
                site.URL = url;
                site.Name = name;
                site.Description = description;
                site.Image = hImage.GenerateImage(thumbDirPath, site);
                sHelper.Update(site);
            }
            return RedirectToAction("SitesList");
        }

        public ActionResult EditSite()
        {
            Guid id;
            if (string.IsNullOrEmpty(Request.Form["id"]) || Request.Form["id"].ToString() == "0")
                id = Guid.NewGuid();
            else
                id = new Guid(Request.Form["id"]);

            var name = Request.Form["name"];
            var description = Request.Form["description"];
            var url = Request.Form["url"];
            var email = Request.Form["email"];
            var firstname = Request.Form["firstname"];
            var lastname = Request.Form["lastname"];
            var actionType = Request.Form["actionType"];

            Guid idCatSelected = new Guid(Request.Form["Key"]);

            var approved = ((Request.Form["approved"] ?? "").Trim().Equals("on", StringComparison.CurrentCultureIgnoreCase));

            using (SiteHelper sHelper = new SiteHelper())
            {
                Site site = sHelper.GetSiteByID(id);
                ImageHelper hImage = new ImageHelper();
                string thumbDirPath = Server.MapPath("~") + "\\App_Data\\thumbnail\\";
                if (actionType.ToUpper() != "DELETE")
                {
                    if (site == null) { site = new Site(); }
                    site.Id = id;
                    site.URL = url;
                    site.Name = name;
                    site.Description = description;
                    site.Approved = approved;
                    site.EMail = email;
                    site.FirstName = firstname;
                    site.LastName = lastname;

                    site.DateCreated = DateTime.Now;
                    //site.Ip = "1.1.1.1";
                    //site.PageRank = 0;                    
                    site.Image = hImage.GenerateImage(thumbDirPath, site);
                    using (CategoryHelper cHelper = new CategoryHelper())
                    {
                        site.Categories.Clear();
                        site.Categories.Add(cHelper.Load(idCatSelected));
                    }
                    sHelper.Update(site);
                }
                else
                {
                    hImage.DeleteImage(thumbDirPath, site);
                    sHelper.Delete(site);
                }
            }
            return RedirectToAction("SitesList");
        }

        /*Settings*/
        [Authorize(Roles = "Administrators")]
        public ActionResult Settings()
        {
            using (SettingHelper sHelper = new SettingHelper())
            {
                ViewData["Settings"] = sHelper.GetAll();         
            }

            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
            ViewData["Title"] = "Settings";

            ViewData["cbCategories"] = new SelectList(AFDSHelper.GetCbItems(true, true), "Key", "Value", AFDSHelper.GetRootCategory().Id);

            return View();
        }

        [Authorize(Roles = "Administrators")]
        public ActionResult EditSettings() {
            var idRootcategory = Request.Form["Rootcategory"];
            var dirname = Request.Form["DirectoryName"];
            var storage = Request.Form["Storage"];
            var theme = Request.Form["Theme"];

            using (SettingHelper sHelper = new SettingHelper()) {
                Hashtable settings = sHelper.GetAll();
                settings["Rootcategory"] = idRootcategory;
                settings["DirectoryName"] = dirname;
                settings["Storage"] = storage;
                settings["Theme"] = theme;
                sHelper.Save(settings);
            }
            return RedirectToAction("Settings");
        }
    }
}
