﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Mvc.Ajax;
using System.Collections;
using AFDS.BusinessLogic;
using AFDS.BusinessLogic.Helpers;
using System.Web.UI.WebControls;
using System.Text.RegularExpressions;
using AFDS.Core;

namespace AFDS_WEB.Controllers
{
    [HandleError]
    public class CatController : Controller
    {
        public ActionResult Index()
        {
            return View();
        }

        public ActionResult View(string slug)
        {
            ViewData["ShowSubmitLink"] = true;
            ViewData["Title"] = AFDSHelper.GetDirectoryName();
            ViewData["HeadTitle"] = ViewData["Title"];

            using (CategoryHelper cHelper = new CategoryHelper())
            {
                Category c = cHelper.GetCategoryBySlug(slug);
                ViewData["Category"] = c;
                ViewData["SubmitCategoryId"] = c.Id.ToString();
            }
            return View();
        }

        public ActionResult Submit(string id)
        {
            if(HttpContext.User.Identity.IsAuthenticated){
                System.Web.Security.MembershipUser 
                    user = System.Web.Security.Membership.GetUser(HttpContext.User.Identity.Name);

                ViewData["attribute"] = "READONLY";
                ViewData["e-mail"] = user.Email;                
            }

            ViewData["ShowSubmitLink"] = false;
            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
            ViewData["Title"] = "Submit your Site";
            ViewData["submittext"] = "Submit";

            if (Request.HttpMethod == "GET")
            {
                if (!string.IsNullOrEmpty(id))
                    ViewData["cbCategories"] = new SelectList(AFDSHelper.GetCbItems(true, false), "Key", "Value", new Guid(id));
                else
                    ViewData["cbCategories"] = new SelectList(AFDSHelper.GetCbItems(true, false), "Key", "Value", Guid.Empty);

            }
            else if (Request.HttpMethod == "POST")
            {

                Guid _id = Guid.NewGuid();
                var name = Request.Form["name"];
                var description = Request.Form["description"];
                var url = Request.Form["url"];
                var email = Request.Form["email"];
                var firstname = Request.Form["firstname"];
                var lastname = Request.Form["lastname"];
                Guid idCatSelected = new Guid(Request.Form["Key"]);
                ViewData["cbCategories"] = new SelectList(AFDSHelper.GetCbItems(true, false), "Key", "Value", idCatSelected);

                if (string.IsNullOrEmpty(name) || name.Length < 1) { ViewData["Message"] = "Name is requerid."; return View(); }
                if (string.IsNullOrEmpty(description) || description.Length < 1) { ViewData["Message"] = "Description is requerid."; return View(); }
                if (string.IsNullOrEmpty(url) || url.Length < 1) { ViewData["Message"] = "URL is requerid."; return View(); }
                if (string.IsNullOrEmpty(email) || email.Length < 1) { ViewData["Message"] = "Email is requerid."; return View(); }

                if (!Regex.IsMatch(url, "http://([\\w-]+\\.)+[\\w-]+(/[\\w- ./?%&=]*)?")) { ViewData["Message"] = "URL address is invalid."; return View(); }
                if (!Regex.IsMatch(email, ".*@.*\\..*")) { ViewData["Message"] = "Email address is invalid."; return View(); }

                using (SiteHelper sHelper = new SiteHelper())
                {
                    Site site = new Site();
                    site.Id = _id;
                    site.URL = url;
                    site.Name = name;
                    site.Description = description;
                    site.Approved = false;
                    site.EMail = email;
                    site.FirstName = firstname;
                    site.LastName = lastname;

                    site.DateCreated = DateTime.Now;
                    site.Ip = Request.UserHostAddress;
                    site.PageRank = 0;

                    site.Image = "";
                    using (CategoryHelper cHelper = new CategoryHelper())
                    {
                        site.Categories.Clear();
                        site.Categories.Add(cHelper.Load(idCatSelected));
                    }
                    sHelper.Insert(site);
                }
                return RedirectToAction("SubmitMeesage");
            }
            return View();
        }


        public ActionResult SubmitMeesage(string message)
        {
            ViewData["ShowSubmitLink"] = false;
            ViewData["HeadTitle"] = AFDSHelper.GetDirectoryName();
            return View();
        }

    }
}
