﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar
/*
Based on Chris Pietschmann's View Engine
http://pietschsoft.com/post/2008/08/Custom-Themes-in-ASPNET-MVC-Updated-for-Preview-5.aspx
*/

using System;
using System.Globalization;
using System.Linq;
using System.Web.Mvc;
using AFDS.Web.Engine;



namespace AFDS.Web.Engine
{
	public class AFDSViewEngine : System.Web.Mvc.WebFormViewEngine
	{
        public AFDSViewEngine()
		{
		
                        base.ViewLocationFormats = new string[] {
                            "~/Themes/{2}/Views/{1}/{0}.aspx",
                            "~/Themes/{2}/Views/{1}/{0}.ascx",
                            "~/Themes/{2}/Views/Shared/{0}.aspx",
                            "~/Themes/{2}/Views/Shared/{0}.ascx"
                        };

                        base.MasterLocationFormats = new string[] {
                            "~/Themes/{2}/Views/{1}/{0}.master",
                            "~/Themes/{2}/Views/Shared/{0}.master"
                        };

                        base.PartialViewLocationFormats = new string[] {
                            "~/Themes/{2}/Views/{1}/{0}.aspx",
                            "~/Themes/{2}/Views/{1}/{0}.ascx",
                            "~/Themes/{2}/Views/Shared/{0}.aspx",
                            "~/Themes/{2}/Views/Shared/{0}.ascx"
                        };          

        }

		protected override IView CreateView(ControllerContext controllerContext, string viewPath, string masterPath)
		{
			return new AFDSView(viewPath, masterPath);
		}

        public override ViewEngineResult FindView(ControllerContext controllerContext, string viewName, string masterName, bool useCache)
		{
			if (controllerContext == null)
				throw new ArgumentNullException("controllerContext");

			if (string.IsNullOrEmpty(viewName))
				throw new ArgumentException("Value is required.", "viewName");

			string themeName = this.GetThemeToUse(controllerContext);

			string[] searchedViewLocations;
			string[] searchedMasterLocations;

			string controllerName = controllerContext.RouteData.GetRequiredString("controller");

			string viewPath = this.GetPath(this.ViewLocationFormats, viewName, controllerName, themeName, out searchedViewLocations);

            string masterPath = this.GetMasterPath(this.MasterLocationFormats, "Site",
                    controllerName, themeName, out searchedMasterLocations);


			//string masterPath = this.GetPath(this.MasterLocationFormats, masterName, controllerName, themeName, out searchedMasterLocations);

			if (!(string.IsNullOrEmpty(viewPath)) && (!(masterPath == string.Empty) || string.IsNullOrEmpty(masterName)))
				return new ViewEngineResult(this.CreateView(controllerContext, viewPath, masterPath), this);
			
			return new ViewEngineResult(searchedViewLocations.Union<string>(searchedMasterLocations));
		}

        public override ViewEngineResult FindPartialView(ControllerContext controllerContext, string partialViewName, bool useCache)
		{
			if (controllerContext == null)
				throw new ArgumentNullException("controllerContext");

			if (string.IsNullOrEmpty(partialViewName))
				throw new ArgumentException("Value is required.", partialViewName);

			string themeName = this.GetThemeToUse(controllerContext);

			string[] searchedLocations;

			string controllerName = controllerContext.RouteData.GetRequiredString("controller");

            string partialPath = this.GetMasterPath(this.PartialViewLocationFormats, partialViewName, controllerName, themeName, out searchedLocations);
            //string partialPath = this.GetPath(this.PartialViewLocationFormats, partialViewName, controllerName, themeName, out searchedLocations);

            

			if (string.IsNullOrEmpty(partialPath))
				return new ViewEngineResult(searchedLocations);

			return new ViewEngineResult(this.CreatePartialView(controllerContext, partialPath), this);
		}

		private string GetThemeToUse(ControllerContext controllerContext)
		{
			string themeName = controllerContext.HttpContext.Items["themeName"] as string;
			if (themeName == null) themeName = "Default";
			return themeName;
		}

		private string GetPath(string[] locations, string viewName, string controllerName, string themeName, out string[] searchedLocations)
		{
			string path = null;

			searchedLocations = new string[locations.Length*2];

			for (int i = 0; i < locations.Length; i++)
			{
				path = string.Format(CultureInfo.InvariantCulture, locations[i], viewName, controllerName, themeName);
				if (this.VirtualPathProvider.FileExists(path))
				{
					searchedLocations = new string[0];
					return path;
				}
				searchedLocations[i] = path;
			}

			//try default theme, for a fallback
			for (int i = 0; i < locations.Length; i++)
			{
				path = string.Format(CultureInfo.InvariantCulture, locations[i], viewName, controllerName, "Default");
				if (this.VirtualPathProvider.FileExists(path))
				{
					searchedLocations = new string[0];
					return path;
				}
				searchedLocations[i + locations.Length] = path;
			}

			return null;
		}

        private string GetMasterPath(string[] locations, string viewName,
                       string controllerName, string themeName, out string[] searchedLocations)
        {
            string path = null;

            searchedLocations = new string[locations.Length];

            for (int i = 0; i < locations.Length; i++)
            {
                path = string.Format(CultureInfo.InvariantCulture, locations[i],
                                     new object[] { viewName, controllerName, themeName });
                if (this.VirtualPathProvider.FileExists(path))
                {
                    searchedLocations = new string[0];
                    return path;
                }
                searchedLocations[i] = path;
            }
            return null;
        }


	}
}
