﻿//## Microsoft Reciprocal License (Ms-RL)
//##
//## This license governs use of the accompanying software. If you use the software, you accept this license. If you do not accept the license, do not use the software.
//##
//## 1. Definitions
//##
//## The terms "reproduce," "reproduction," "derivative works," and "distribution" have the same meaning here as under U.S. copyright law.
//##
//## A "contribution" is the original software, or any additions or changes to the software.
//##
//## A "contributor" is any person that distributes its contribution under this license.
//## 
//## "Licensed patents" are a contributor's patent claims that read directly on its contribution.
//## 
//## 2. Grant of Rights
//## 
//## (A) Copyright Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free copyright license to reproduce its contribution, prepare derivative works of its contribution, and distribute its contribution or any derivative works that you create.
//## 
//## (B) Patent Grant- Subject to the terms of this license, including the license conditions and limitations in section 3, each contributor grants you a non-exclusive, worldwide, royalty-free license under its licensed patents to make, have made, use, sell, offer for sale, import, and/or otherwise dispose of its contribution in the software or derivative works of the contribution in the software.
//## 
//## 3. Conditions and Limitations
//## 
//## (A) Reciprocal Grants- For any file you distribute that contains code from the software (in source code or binary format), you must provide recipients the source code to that file along with a copy of this license, which license will govern that file. You may license other files that are entirely your own work and do not contain code from the software under any terms you choose.
//## 
//## (B) No Trademark License- This license does not grant you rights to use any contributors' name, logo, or trademarks.
//## 
//## (C) If you bring a patent claim against any contributor over patents that you claim are infringed by the software, your patent license from such contributor to the software ends automatically.
//## 
//## (D) If you distribute any portion of the software, you must retain all copyright, patent, trademark, and attribution notices that are present in the software.
//## 
//## (E) If you distribute any portion of the software in source code form, you may do so only under this license by including a complete copy of this license with your distribution. If you distribute any portion of the software in compiled or object code form, you may only do so under a license that complies with this license.
//## 
//## (F) The software is licensed "as-is." You bear the risk of using it. The contributors give no express warranties, guarantees or conditions. You may have additional consumer rights under your local laws which this license cannot change. To the extent permitted under your local laws, the contributors exclude the implied warranties of merchantability, fitness for a particular purpose and non-infringement.  

//Written by Fabian Figueredo
//http://oberdata.com.ar info@fabianfigueredo.com.ar

using System;
using System.Configuration;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Web;
using System.Web.Mvc;
using System.Web.Routing;


namespace AFDS_WEB
{
    // Note: For instructions on enabling IIS6 or IIS7 classic mode, 
    // visit http://go.microsoft.com/?LinkId=9394801

    public class MvcApplication : System.Web.HttpApplication
    {
        public static void RegisterRoutes(RouteCollection routes)
        {
            if(ConfigurationManager.AppSettings["RoutesFor"].ToLower() == "iis6"){

                routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

                routes.MapRoute(
                    "Thanks",
                    "site/thanks/",
                    new { controller = "Cat", action = "Thanks" }
                );

                routes.MapRoute(
                    "Submit",
                    "submit.aspx/{id}/",
                    new { controller = "Cat", action = "Submit", id = Guid.Empty.ToString() }
                );

                routes.MapRoute(
                    "Category",
                    "category.aspx/{slug}",
                    new { controller = "Cat", action = "View", slug = "0" }
                );

                routes.MapRoute(
                    "Default",                                              // Route name
                    "{controller}.aspx/{action}/{id}",                           // URL with parameters
                    new { controller = "Home", action = "Index", id = "" }  // Parameter defaults
                );

                routes.MapRoute(
                    "Admin",
                    "admin.aspx/{action}/{type}/{id}",
                    new { controller = "AFDSAdmin", action = "Index", type = "", id = "0" }
                );            
            }
            else{
                routes.IgnoreRoute("{resource}.axd/{*pathInfo}");

                routes.MapRoute(
                    "Thanks",
                    "site/thanks/",
                    new { controller = "Cat", action = "Thanks" }
                );

                routes.MapRoute(
                    "Submit",
                    "submit/{id}/",
                    new { controller = "Cat", action = "Submit", id = Guid.Empty.ToString() }
                );

                routes.MapRoute(
                    "Category",
                    "category/{slug}/",
                    new { controller = "Cat", action = "View", slug = "0" }
                );

                routes.MapRoute(
                    "Default",                                              // Route name
                    "{controller}/{action}/{id}",                           // URL with parameters
                    new { controller = "Home", action = "Index", id = "" }  // Parameter defaults
                );

                routes.MapRoute(
                    "Admin",
                    "admin/{action}/{type}/{id}",
                    new { controller = "AFDSAdmin", action = "Index", type = "", id = "0" }
                );                        
            }

        }

        protected void Application_Start()
        {
            //Register the routes needed for MVC
            RegisterRoutes(RouteTable.Routes);

            // Replace the Default WebFormViewEngine with out custom WebFormThemeViewEngine
            ViewEngines.Engines.Clear();
            ViewEngines.Engines.Add(new AFDS.Web.Engine.AFDSViewEngine());

            //Load App_Code Class by Reflection
            if (System.Web.Compilation.BuildManager.CodeAssemblies != null) {
                foreach (
                    System.Reflection.Assembly assembly 
                        in 
                            System.Web.Compilation.BuildManager.CodeAssemblies) {
                                var types = assembly.GetExportedTypes();
                                if (types == null || types.Length == 0)
                                    continue;

                                foreach (Type type in types) {
                                    Activator.CreateInstance(type);
                    }
                }
            }
        }


        protected void Application_Error(object sender, EventArgs e) {
            AFDS.Core.AFDSException objError =
                (AFDS.Core.AFDSException)Server.GetLastError().GetBaseException();

            string err = "Error Caught in Application_Error event <br>" +
                    "Error in: " + Request.Url.ToString() +
                    "<br>Go: <a href='/afdsadmin'>Administration Home</a>" +
                    "<br>Error Message:" + objError.CustomMessage +
                    "<br><br>Stack Trace:" + objError.StackTrace.ToString();

            Response.Write(err);
            Server.ClearError();
            //additional actions...
        } 

        protected void Application_BeginRequest(Object Sender, EventArgs e)
        {
            SetTheme();
        }

        private void SetTheme()
        {
            //set the theme for the ViewEngine to utilize.
            HttpContext context = HttpContext.Current;
            if (context.Items.Contains("theme")) context.Items.Remove("theme");
            context.Items.Add(
                "themeName",
                CheckTheme(context.Request.QueryString["theme"])
                    ?? CheckTheme(context.Request.Form["theme"])
                    ?? CheckTheme(context.Request.Headers["theme"])
                    ?? CheckTheme((context.Request.Cookies["theme"] == null) ? "" : context.Request.Cookies["theme"].Value)
                    ?? CheckTheme((context.Session == null) ? "" : context.Session["theme"] as string)
                    //?? CheckTheme(ConfigurationSettings.AppSettings["theme"])
                    ?? CheckTheme(AFDS.BusinessLogic.Helpers.AFDSHelper.GetThemeName())
                    ?? "Default"
            );
        }
        private string CheckTheme(string themeName)
        {
            if (!string.IsNullOrEmpty(themeName))
            {
                var path = HttpContext.Current.Server.MapPath(string.Format(
                    "~/Themes/{0}",
                    themeName
                ));
                if (Directory.Exists(path))
                    return themeName;
            }
            return null;
        }
        private string CheckTheme(HttpCookie cookie)
        {
            if (cookie != null)
                return CheckTheme(cookie.Value);
            return null;
        }
    }
}